@@ -3,9 +3,11 @@ from pathlib import Path
 import shutil
 import json
 import os
+from os.path import abspath
 
 from onnc.bench.core.model.model import Model
 
+
 class Deployment:
     """
     Deployment is a container class for built artifacts and reports
@@ -59,16 +61,50 @@ class Deployment:
                 return files
 
         return []
-
     @property
     def loadable(self):
-        if os.path.exists(self.base_path):
-            model_src = self.base_path / Path('build')
-            if model_src.exists():
-                for i in os.listdir(model_src):
-                    if i.lower().startswith('model'):
-                        return Model(os.path.join(model_src, i))
-        raise Exception('Unable to get loadable.')
+        if len(self.loadables) > 1:
+            raise ValueError("Error: Compilation result contains multiple "
+                             "models, use loadables instead.")
+        elif len(self.loadables) == 0:
+            raise ValueError("Error: No loadable found "
+                             "(Maybe compilation fail?)")
+        return self.loadables[0]
+
+    @property
+    def loadables(self) -> List[Model]:
+
+        if not os.path.exists(self.base_path):
+            raise Exception(f'Deployment base_path is not a directory: '
+                            f'{self.base_path}.')
+        res = []
+        for sub_model_root in os.listdir(self.base_path):
+            root = self.base_path / sub_model_root
+            if not os.path.isdir(root / "build"):
+                raise Exception(f'Invalid deployment subdir {root}: '
+                                f'Missing model directory')
+            model_to_add = None
+            if os.path.isdir(root / "build/model"):
+                """
+                Case 1: submodel is a directory named model contains
+                        multiple files. Ex: Openvino .xml, .bin
+                """
+                model_to_add = Model(abspath(root / "build/model"))
+            else:
+                """
+                Case 2: submodel is a single file name starts with model
+                """
+                for file in os.listdir(root / "build"):
+                    if file.startswith("model"):
+                        model_to_add = Model(abspath(root / f"build/{file}"))
+                        break
+            if not model_to_add:
+                raise Exception(
+                    f"Unrecognized deployment subdir "
+                    f"abspath{root}, plz contacts developers to fix it.")
+            else:
+                res.append(model_to_add)
+        return res
 
     @property
     def compile_logs(self):
@@ -83,11 +119,12 @@ class Deployment:
 
     @property
     def meta(self):
-        return {"base_path": str(self.base_path),
-                "compiled_files": [str(x) for x in self.compiled_files],
-                "report_path": str(self.report_path),
-                "report": self.report
-                }
+        return {
+            "base_path": str(self.base_path),
+            "compiled_files": [str(x) for x in self.compiled_files],
+            "report_path": str(self.report_path),
+            "report": self.report
+        }
 
     def save(self):
         _path = self.base_path / self.META_FNAME
