@@ -12,6 +12,8 @@ from .model import Model, Tensor
 from . import ModelFormat, ModelDataType
 from .identifier import identify
 from ..common import get_tmp_path
+from importlib import import_module
+import numpy as np
 
 
 class MetadataRetriverRegistry(type):
@@ -40,90 +42,68 @@ class MetadataRetriver(metaclass=MetadataRetriverRegistry):
         return identify(model) == cls.FORMAT
 
     @abstractmethod
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
-        raise NotImplementedError("`retrive_inputs` has to be implemented")
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
+        raise NotImplementedError("`retrieve_inputs` has to be implemented")
 
     @abstractmethod
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
-        raise NotImplementedError("`retrive_outputs` has to be implemented")
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
+        raise NotImplementedError("`retrieve_outputs` has to be implemented")
 
-    def retrive(self, model: Model) -> ModelMeta:
-        inputs = self.retrive_inputs(model)
+    def retrieve(self, model: Model) -> ModelMeta:
+        inputs = self.retrieve_inputs(model)
         logger.debug(f"Inputs of {model.name}: {inputs}")
-        outputs = self.retrive_outputs(model)
+        outputs = self.retrieve_outputs(model)
         logger.debug(f"Outputs of {model.name}: {outputs}")
         return ModelMeta(inputs=inputs, outputs=outputs)
 
 
-def metadataretriver_selector(model: Model) -> MetadataRetriver:
-    for metadataretriver in MetadataRetriver.REGISTRY:
-        if metadataretriver.is_me(model):
-            return metadataretriver
-    raise NotImplementedError(f"Unalble to retrive metadata from {model}")
+def retrieve_model_metadata(model: Model) -> ModelMeta:
+    for metadataretriever in MetadataRetriver.REGISTRY:
+        if metadataretriever.is_me(model):
+            return metadataretriever().retrieve(model)
 
-
-def retrive_model_metadata(model: Model) -> ModelMeta:
-    for metadataretriver in MetadataRetriver.REGISTRY:
-        if metadataretriver.is_me(model):
-            return metadataretriver().retrive(model)
-
-    raise NotImplementedError(f"Unalble to retrive metadata of {model.format}")
-
-
-# class FileSerializer(MetadataRetriver):
-
-#     FORMAT: ModelFormat = ModelFormat.NON_SPECIFIED
-
-#     def transform(self, model: Model):
-#         dest = self.get_param('dest')
-
-#         if model.src.is_file():
-#             shutil.move(model.src, dest)
-
-#         elif model.src.is_dir():
-#             # Zip the dir if the model is in dir form
-#             tmp_path = Path(get_tmp_path())
-#             shutil.make_archive(tmp_path, 'zip', model.src)
-#             shutil.move(tmp_path / '.zip', dest)
-
-#         return Model(dest)
+    raise NotImplementedError(f"Unalble to retrieve metadata of {model.format}")
 
 
 class H5(MetadataRetriver):
 
     FORMAT = ModelFormat.H5
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    from .dtype_map import TF_map
+
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         from tensorflow import keras  # type: ignore[import]
         _model = keras.models.load_model(model.src)
 
-        return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in _model.inputs
-        ]
+        res = []
+        for x in _model.inputs:
+            shape = tuple(-1 if not s else s for s in x.shape)
+            res.append(
+                Tensor(name=x.name, shape=shape,
+                       dtype=self.TF_map.map(x.dtype)))
+        return res
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
-        # from tensorflow import keras
-        # model = keras.models.load_model(model.src)
-        # outputs = {}
-        # for x in model.outputs:
-        #     inputs[x.name] = Tensor(name=x.name,
-        #                             shape=tuple(x.shape.as_list()),
-        #                             dtype=x.dtype)
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         from tensorflow import keras  # type: ignore[import]
         _model = keras.models.load_model(model.src)
 
-        return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in _model.outputs
-        ]
+        res = []
+        for x in _model.outputs:
+            shape = tuple(-1 if not s else s for s in x.shape)
+            res.append(
+                Tensor(name=x.node.outbound_layer.name,
+                       shape=shape,
+                       dtype=self.TF_map.map(x.dtype)))
+
+        return res
 
 
 class ONNX(MetadataRetriver):
 
     FORMAT = ModelFormat.ONNX
+    from .dtype_map import ONNX_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         import onnx
         onnx_model = onnx.load(model.src)
 
@@ -132,52 +112,68 @@ class ONNX(MetadataRetriver):
         net_feed_input = list(set(input_all) - set(input_initializer))
 
         res = []
+
         for input in onnx_model.graph.input:
+
+            shape = tuple(
+                xx.dim_value for xx in input.type.tensor_type.shape.dim)
+            shape = tuple(-1 if int(s) == 0 else s for s in shape)
+
             if input.name in net_feed_input:
                 res.append(
-                    Tensor(
-                        input.name,
-                        tuple([
-                            xx.dim_value
-                            for xx in input.type.tensor_type.shape.dim
-                        ]), ModelDataType.NON_SPECIFIED))
+                    Tensor(input.name, shape,
+                           self.ONNX_map.map(input.type.tensor_type.elem_type)))
 
         return res
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         import onnx
         model = onnx.load(model.src)
-        output = [node.name for node in model.graph.output]
-        return [Tensor(x, tuple(), ModelDataType.NON_SPECIFIED) for x in output]
+        res = []
+        for model_output in model.graph.output:
+            shape = tuple(
+                s.dim_value for s in model_output.type.tensor_type.shape.dim)
+            shape = tuple(-1 if int(s) == 0 else s for s in shape)
+            res.append(
+                Tensor(
+                    model_output.name, shape,
+                    self.ONNX_map.map(model_output.type.tensor_type.elem_type)))
+        return res
 
 
 class PTH(MetadataRetriver):
 
     FORMAT = ModelFormat.PTH
+    from .dtype_map import NP_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         import torch
-        _model = torch.jit.load(model.src)
+        _model = torch.load(model.src)
         tensor = list(_model.parameters())[  # type: ignore[union-attr]
             0].detach()
         t_name = "input" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tensor.shape, self.NP_map.map(tensor.dtype.type))
+        ]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         import torch
-        _model = torch.jit.load(model.src)
+        _model = torch.load(model.src)
 
         # type: ignore[union-attr]
         tensor = list(_model.parameters())[-1].detach()
         t_name = "output" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tensor.shape, self.NP_map.map(tensor.dtype.type))
+        ]
 
 
 class PB(MetadataRetriver):
 
     FORMAT = ModelFormat.PB
+    from .dtype_map import TF_map
 
     # https://stackoverflow.com/questions/43517959/given-a-tensor-flow-model-graph-how-to-find-the-input-node-and-output-node-name
     def _load_graph(self, frozen_graph_filename):
@@ -204,7 +200,7 @@ class PB(MetadataRetriver):
         outputs = list(outputs_set)
         return (inputs, outputs)
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         import tensorflow as tf
         ops = self._load_graph(model.src)
         inputs = []
@@ -214,15 +210,15 @@ class PB(MetadataRetriver):
 
         return [
             Tensor(name=x.name,
-                   shape=tuple(x.outputs[0].shape.as_list()),
-                   dtype=x.outputs[0].dtype) for x in inputs
+                   shape=tuple(xx if xx else -1 for xx in x.outputs[0].shape),
+                   dtype=self.TF_map.map(x.outputs[0].dtype)) for x in inputs
         ]
         """
         >>> [x.size for x in graph_def.node[0].attr['shape'].shape.dim]
         [-1, 96, 96, 3]
         """
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         import tensorflow as tf
         ops = self._load_graph(model.src)
         inputs = []
@@ -240,8 +236,8 @@ class PB(MetadataRetriver):
 
         return [
             Tensor(name=x.name,
-                   shape=tuple(x.outputs[0].shape.as_list()),
-                   dtype=x.outputs[0].dtype) for x in outputs
+                   shape=tuple(xx if xx else -1 for xx in x.outputs[0].shape),
+                   dtype=self.TF_map.map(x.outputs[0].dtype)) for x in outputs
         ]
 
 
@@ -249,83 +245,71 @@ class SavedModel(MetadataRetriver):
 
     FORMAT = ModelFormat.SAVED_MODEL
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    from .dtype_map import TF_map
+
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
+        assert type(model) == Model
         from tensorflow import keras  # type: ignore[import]
         _model = keras.models.load_model(model.src)
-
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in _model.inputs
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1 for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype)) for x in _model.inputs
         ]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
-        # from tensorflow import keras
-        # model = keras.models.load_model(model.src)
-        # outputs = {}
-        # for x in model.outputs:
-        #     inputs[x.name] = Tensor(name=x.name,
-        #                             shape=tuple(x.shape.as_list()),
-        #                             dtype=x.dtype)
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
+
         from tensorflow import keras  # type: ignore[import]
         _model = keras.models.load_model(model.src)
 
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1
+                               for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype))
+            for x in _model.inputs
             for x in _model.outputs
         ]
 
 
-class ZippedSavedModel(MetadataRetriver):
+class ZippedSavedModel(SavedModel):
 
     FORMAT = ModelFormat.ZIPPED_SAVED_MODEL
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
-        from tensorflow import keras  # type: ignore[import]
-
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         temp = get_tmp_path()
 
         with zipfile.ZipFile(model.src,
                              'r') as zip_ref:  # type: ignore[arg-type]
             zip_ref.extractall(temp)
+        return super().retrieve_inputs(Model(temp))
 
-        _model = keras.models.load_model(temp)
-
-        return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in _model.inputs
-        ]
-
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
-        from tensorflow import keras  # type: ignore[import]
-
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         temp = get_tmp_path()
-
         with zipfile.ZipFile(model.src,
                              'r') as zip_ref:  # type: ignore[arg-type]
             zip_ref.extractall(temp)
-
-        _model = keras.models.load_model(temp)
-
-        return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in _model.outputs
-        ]
+        return super().retrieve_outputs(Model(temp))
 
 
 class TFKerasModel(MetadataRetriver):
 
     FORMAT = ModelFormat.TF_KERAS_MODEL
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    from .dtype_map import TF_map
+
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in model.src.inputs
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1 for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype)) for x in model.src.inputs
         ]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in model.src.outputs
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1 for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype)) for x in model.src.outputs
         ]
 
 
@@ -335,17 +319,20 @@ class KerasModel(MetadataRetriver):
     '''
 
     FORMAT = ModelFormat.KERAS_MODEL
+    from .dtype_map import TF_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in model.src.inputs
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1 for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype)) for x in model.src.inputs
         ]  # type: ignore[union-attr]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         return [
-            Tensor(name=x.name, shape=tuple(x.shape.as_list()), dtype=x.dtype)
-            for x in model.src.outputs
+            Tensor(name=x.name,
+                   shape=tuple(xx if xx else -1 for xx in x.shape),
+                   dtype=self.TF_map.map(x.dtype)) for x in model.src.outputs
         ]  # type: ignore[union-attr]
 
 
@@ -353,39 +340,50 @@ class PytorchModel(MetadataRetriver):
     """Use python MRO to check if it contains specific str"""
 
     FORMAT = ModelFormat.PT_NN_MODULE
+    from .dtype_map import NP_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         _model = model.src
         tensor = list(_model.parameters())[  # type: ignore[union-attr]
             0].detach()
         t_name = "input" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tuple(x if x else -1 for x in tensor.shape),
+                   self.NP_map.map(tensor.dtype.type))
+        ]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         _model = model.src
 
         # type: ignore[union-attr]
         tensor = list(_model.parameters())[-1].detach()
         t_name = "output" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tuple(x if x else -1 for x in tensor.shape),
+                   self.NP_map.map(tensor.dtype.type))
+        ]
 
 
 class TorchTracedModel(MetadataRetriver):
 
     FORMAT = ModelFormat.TORCH_TRACED
+    from .dtype_map import NP_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
         import torch
         _model = torch.jit.load(model.src)
         tensor = list(_model.parameters())[  # type: ignore[union-attr]
             0].detach()
         t_name = "input" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tuple(x if x else -1 for x in tensor.shape),
+                   self.NP_map.map(tensor.dtype.type))
+        ]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         import torch
         _model = torch.jit.load(model.src)
 
@@ -393,15 +391,20 @@ class TorchTracedModel(MetadataRetriver):
         tensor = list(_model.parameters())[-1].detach()
         t_name = "output" if tensor.name == None else tensor.name
         tensor = tensor.numpy()
-        return [Tensor(t_name, tensor.shape, tensor.dtype)]
+        return [
+            Tensor(t_name, tuple(x if x else -1 for x in tensor.shape),
+                   self.NP_map.map(tensor.dtype.type))
+        ]
 
 
 class TFLiteModel(MetadataRetriver):
     """Use python MRO to check if it contains specific str"""
 
     FORMAT = ModelFormat.TFLITE
+    from .dtype_map import NP_map
+    _dtype_map = NP_map
 
-    def retrive_inputs(self, model: Model) -> List[Tensor]:
+    def retrieve_inputs(self, model: Model) -> List[Tensor]:
 
         try:
             import tflite_runtime.interpreter as tflite
@@ -409,10 +412,16 @@ class TFLiteModel(MetadataRetriver):
         except ModuleNotFoundError:
             import tensorflow as tf
             interpreter = tf.lite.Interpreter(model_path=str(model.src))
-        return [Tensor(i["name"], i["shape"].tolist(), i['dtype'])
-                for i in interpreter.get_input_details()]
 
-    def retrive_outputs(self, model: Model) -> List[Tensor]:
+        res = []
+        for i in interpreter.get_input_details():
+            res.append(
+                Tensor(i["name"],
+                       tuple(-1 if not s else int(s) for s in i["shape"]),
+                       self._dtype_map.map(i['dtype'])))
+        return res
+
+    def retrieve_outputs(self, model: Model) -> List[Tensor]:
         try:
             import tflite_runtime.interpreter as tflite
             interpreter = tflite.Interpreter(model_path=str(model.src))
@@ -420,5 +429,10 @@ class TFLiteModel(MetadataRetriver):
             import tensorflow as tf
             interpreter = tf.lite.Interpreter(model_path=str(model.src))
 
-        return [Tensor(i["name"], i["shape"].tolist(), i['dtype'])
-                for i in interpreter.get_output_details()]
+        res = []
+        for i in interpreter.get_output_details():
+            res.append(
+                Tensor(i["name"],
+                       tuple(-1 if not s else int(s) for s in i["shape"]),
+                       self._dtype_map.map(i['dtype'])))
+        return res
