@@ -1,5 +1,5 @@
 from pathlib import Path
-from typing import Dict, Union, List, Tuple
+from typing import Dict, Union, List, Tuple, Optional
 import os
 import json
 import shutil
@@ -13,13 +13,14 @@ from .core.model import ModelFormat, TF_MODEL_FORMATS, ModelDataType
 from .core.model.model import Tensor
 
 from .core.dataset import DatasetFormat
-from .core.dataset.layout import DataLayout
-from .core.project import ProjectData
+
 from .core.compiler.onnc_saas import ONNCSaaSBuilder
 # from .core.compiler.nnuxe import NNUXEBuilder
 # from .config import api_protocol, api_url, api_port
 from .core.common import get_tmp_path, get_temp_base
+
 from .config import default_builder, default_builder_params
+from onnc.bench.core.model.model import Model
 
 
 def login(api_key: str, password: str = ""):
@@ -47,31 +48,6 @@ class Project:
 
             logger.info(f"ProjectID: {self.builder._project_id}")
 
-    @classmethod
-    def get_model_meta(cls, model_package, inputs_as_nchw):
-        model_meta = {
-            "inputs": [x.dump() for x in model_package.model.inputs],
-            "inputs_as_nchw": inputs_as_nchw,
-            "outputs": [x.dump() for x in model_package.model.outputs],
-            "format": model_package.model.format.name
-        }
-
-        return model_meta
-
-    @classmethod
-    def get_dataset_meta(cls, model_package):
-        if not model_package.dataset:
-            return
-        dataset_meta = {
-            "shape": model_package.dataset.shape,
-            "format": model_package.dataset.format.name
-        }
-
-        if isinstance(model_package.dataset.layout, DataLayout):
-            dataset_meta["layout"] = model_package.dataset.layout.dump()
-
-        return dataset_meta
-
     def add_model(
             self,
             model: Union[object, str, Path],
@@ -82,7 +58,8 @@ class Project:
             model_inputs: List[Union[List, Tuple, Tensor]] = [],
             # [name: str, shape:Tuple, dtype:ModelDataType]
             model_outputs: List[Union[List, Tuple, Tensor]] = [],
-            inputs_as_nchw: Union[str, bool, None, List[str]] = None):
+            inputs_as_nchw: Union[str, bool, None, List[str]] = None,
+            batch_dim: Optional[int] = 0):
         """Add a model and its corresponding calibration samples.
 
         :param Union[object, str, Path] model:
@@ -126,26 +103,17 @@ class Project:
 
         """
         model_package = ModelPackage(model, samples, model_format,
-                                     sample_format, model_inputs, model_outputs)
+                                     sample_format, model_inputs, model_outputs,
+                                     batch_dim)
+        serialized_model, serialized_dataset = model_package.serialize()
 
-        model_tmp_path = Path(get_tmp_path())
-        dataset_tmp_path = Path(get_tmp_path())
+        model_id = self.builder.prepare_model(serialized_model,
+                                              serialized_dataset,
+                                              inputs_as_nchw)
+        self.builder.set_compilations_params(model_id,compile_params={},
+            calibrator_params={})
 
-        model_package.serialize(model_tmp_path, dataset_tmp_path)
-
-        model_meta = self.get_model_meta(model_package, inputs_as_nchw)
-        dataset_meta = self.get_dataset_meta(model_package)
-
-        model_id = self.builder.prepare_model(model_package.model.src,
-                                              model_package.dataset.src,
-                                              model_meta=model_meta,
-                                              dataset_meta=dataset_meta)
-
-        self.builder.calibrate(model_id, params={})
-        self.builder.compile(model_id, params={})
-        return model_meta
-
-    def compile(self, target: str, **params):
+    def compile(self, target: str, converter_params: Dict = None):
         """Trigger the compilation process and transform the given
         model into C function calls.
 
@@ -162,12 +130,15 @@ class Project:
             more details.
 
         """
+        if converter_params is None:
+            converter_params = {}
         target_id = self.builder.get_device_id(target)
-        res = self.builder.build(target_id)
+        res = self.builder.build(target_id, converter_params)
 
         if isinstance(self.builder, ONNCSaaSBuilder):
-            logger.info(f"You review compilation report at https://app.onnc.skymizer.com"
-                        f"/app/builds/{res['id']}")
+            logger.info(
+                f"You review compilation report at https://app.onnc.skymizer.com"
+                f"/app/builds/{res['id']}")
 
         return res
 
@@ -184,7 +155,6 @@ class Project:
         """
         if isinstance(path, str):
             path = Path(path)
-
         return self.builder.save(path)
 
     def prune(self, params={}):
